/*
	ByteSplit
	(c) 2003 by Don K. Eitner

	Based on DESplit 0.7, also (c) 2003 by Don K. Eitner
*/

/* Load IBM Rexx utility functions */
CALL RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
CALL SysLoadFuncs

/* Setup ANSI codes for display properties */
CALL AnsiSetup

/* Setup global variables */
ProgVersion = 0.94
OverWarning = 1
Verify = 1

/* Get disc size and directory tree to backup from user input */
PARSE ARG CmdLine
CALL ParseInput

/***********************************************************/
/* Mode could be TESTSPLIT, SPLIT, TESTJOIN or JOIN        */
/* NOTE:  Currently only TESTSPLIT and SPLIT are supported */
/***********************************************************/
SELECT
	WHEN Translate(Mode) = 'TESTSPLIT' THEN
		DO
			CALL GuessChunks
			SAY 'Number of chunks to create:' NumChunks
		END
	WHEN Translate(Mode) = 'SPLIT' THEN
		CALL SplitUp
/*
	WHEN Translate(Mode) = 'JOIN' THEN
		CALL JoinUp
*/
	OTHERWISE
		NOP
END

EXIT


AnsiSetup:
	Esc=d2c(27)d2c(91)

	/* Attributes */
	Hig=Esc'1m'
	Non=Esc'0m'

	/* Foreground Colors */
	Bla=Esc'30m'
	Red=Esc'31m'
	Gre=Esc'32m'
	Bro=Esc'33m'
	Blu=Esc'34m'
	Pur=Esc'35m'
	Tur=Esc'36m'
	Whi=Esc'37m'
	LBla=Hig||Bla
	LRed=Hig||Red
	LGre=Hig||Gre
	LBro=Hig||Bro
	LBlu=Hig||Blu
	LPur=Hig||Pur
	LTur=Hig||Tur
	LWhi=Hig||Whi

	/* Background Colors */
	BBla=Esc'40m'
	BRed=Esc'41m'
	BGre=Esc'42m'
	BBro=Esc'43m'
	BBlu=Esc'44m'
	BCya=Esc'45m'
	BTur=Esc'46m'
	BWhi=Esc'47m'
RETURN


ParseInput:
	IF CmdLine = '' THEN
		CALL GiveHelp

	DO WHILE CmdLine <> ''
		PARSE VAR CmdLine CL1 CmdLine
		IF Left(CL1, 1) = '/' | Left(CL1, 1) = '-' THEN
			DO
				CL1 = Translate(CL1)
				SELECT
					WHEN Substr(CL1, 2) = 'S' THEN
						DO
							Mode = 'SPLIT'
						END
					WHEN Substr(CL1, 2) = 'J' THEN
						DO
							Mode = 'JOIN'
						END
					WHEN Substr(CL1, 2) = 'TS' THEN
						DO
							Mode = 'TESTSPLIT'
						END
					WHEN Substr(CL1, 2) = 'TJ' THEN
						DO
							Mode = 'TESTJOIN'
						END
					WHEN Substr(CL1, 2) = 'C' THEN   /* get the maximum output chunk size */
						DO
							PARSE VAR CmdLine CL1 CmdLine
							ChunkSize = CL1
						END
					WHEN Substr(CL1, 2) = 'U' THEN   /* get unit of measure in which to work */
						DO
							PARSE VAR CmdLine CL1 CmdLine
							Unit = Translate(CL1)
						END
					WHEN Substr(CL1, 2) = 'I' THEN   /* get input filename */
						DO
							IF Left(CmdLine, 1) = '"' THEN
								DO
									PARSE VAR CmdLine WITH '"' CL1 '"' CmdLine
									InFile = CL1
								END
							ELSE
								DO
									PARSE VAR CmdLine CL1 CmdLine
									InFile = CL1
								END
						END
					WHEN (Substr(CL1, 2) = 'V') | (Substr(CL1, 2) = 'VERSION') THEN   /* show program version */
						DO
							SAY ProgVersion
							EXIT
						END
					OTHERWISE
						CALL GiveHelp
				END
			END
		ELSE
			DO
				CALL GiveHelp
				EXIT
			END
	END

	SELECT
		WHEN Unit = 'KB' THEN
			ChunkSize = ChunkSize * 1024
		WHEN Unit = 'MB' THEN
			ChunkSize = ChunkSize * 1024**2
		WHEN Unit = 'GB' THEN
			ChunkSize = ChunkSize * 1024**3
		OTHERWISE
			NOP
	END

	RC = SysFileTree(InFile, 'File', 'F')
	PARSE VAR File.1 Date Time Size Attribs Filename
	Filename = Strip(Filename, 'B', ' ')
RETURN


GiveHelp:
	CALL SysCls
	SAY LTur||'ByteSplit' ProgVersion '(c) 2003 by Don Eitner'||Non
	SAY
	SAY LGre||'Usage:'||Non
	SAY '  ByteSplit [-(S|J|TS|TJ) -C ChunkSize -U Unit -I InputFile] [options]'
	SAY
	SAY LWhi||'Only one of the following modes may be given'||Non
	SAY '-S  : byte splits InputFile based on ChunkSize and Unit'
	SAY '-TS : only tells how many chunks will be created if you split InputFile'
	SAY '-J  : joins sequentially numbered input files (eg. filename.001, filename.002)'
	SAY '-TJ : only tests how many sequentially numbered input files can be joined'
	SAY
	SAY 'ChunkSize must be a positive value and determines the maximum size (see'
	SAY 'Unit below) of each output chunk.'
	SAY
	SAY 'Unit can be KB, MB or GB and affects ChunkSize on a scale of 1024 (ie.'
	SAY 'ChunkSize of 10 and Unit of MB will count 10240 KBytes not 10000 KBytes).'
	SAY
	SAY 'InputFile must be enclosed with quotation marks if it contains spaces.'
	SAY
	SAY LGre||'Available Options:'||Non
	SAY '  -V = show ByteSplit version'
	SAY '  -? = show this help screen'
EXIT


GuessChunks:
	IF Size / ChunkSize > Size % ChunkSize THEN
		NumChunks = Size % ChunkSize + 1
	ELSE
		NumChunks = Size % ChunkSize

	IF NumChunks > 999 THEN
		DO
			SAY 'This operation will require more than 999 chunks to be created!'
			SAY 'I cannot do that (at least not in this version).'
			NumChunks = 0
		END
RETURN


SplitUp:
	Num = 0
	OutSize = 0

	CALL GuessChunks
	IF NumChunks < 2 THEN
		RETURN

	RC = Stream(Filename, 'c', 'open')

	DO WHILE Chars(Filename) > 0
		Num = Num + 1
		OutNum = Right(Num, 3, '0')
		OutFile = Filename||'.'||OutNum
		Proceed = 1
		IF OverWarning = 1 THEN
			CALL Warn
		IF Proceed = 1 THEN
			DO
				RC = SysFileDelete(OutFile)

				IF Num < NumChunks THEN
					DO
						SmallChunks = Left(ChunkSize, 1) * 10
						SmallOutSize = 0
						SmallChunkSize = Trunc(ChunkSize / SmallChunks)
						DO X = 1 TO SmallChunks
							CALL Charout OutFile, Charin(Filename, , SmallChunkSize)
							SmallOutSize = SmallOutSize + SmallChunkSize
							OutSize = OutSize + SmallChunkSize
							SAY 'Bytesplit Progress:' Format(OutSize / Size * 100, , 0)||'%'
						END
						IF SmallOutSize < ChunkSize THEN
							DO
								SizeDiff = ChunkSize - SmallOutSize
								CALL Charout OutFile, Charin(Filename, , SizeDiff)
								OutSize = OutSize + SizeDiff
								SAY 'Bytesplit Progress:' Format(OutSize / Size * 100, , 0)||'%'
							END
					END
				ELSE
					DO
						SizeDiff = Size - OutSize
						CALL CharOut OutFile, CharIn(Filename, , SizeDiff)
						OutSize = OutSize + SizeDiff
						SAY 'Bytesplit Progress:' Format(OutSize / Size * 100, , 0)||'%'
					END

				RC = Stream(OutFile, 'c', 'close')
				IF RC <> 'READY:' THEN
					SAY 'Error! RC =' RC
			END
		ELSE
			DO
				RC = Stream(Filename, 'c', 'seek +'||ChunkSize)
				OutSize = OutSize + Stream(OutFile, 'c', 'query size')
				SAY 'Bytesplit Progress:' Format(OutSize / Size * 100, , 0)||'%'
			END
	END
	RC = Stream(Filename, 'c', 'close')
	IF RC <> 'READY:' THEN
		SAY 'Error! RC =' RC
	IF Num <> NumChunks THEN
		SAY 'Unmatched NumChunks calculated.' '0d0a'x 'Num =' Num 'NumChunks =' NumChunks
	IF Verify = 1 THEN
		CALL VerifyOutput 'split'
RETURN

/*
JoinUp:
	NumChunks = CountChunks()
	InSize = 0
	Proceed = 1
	IF OverWarning = 1 THEN
		CALL Warn
	IF Proceed = 1 THEN
		DO
			RC = SysFileDelete(OutFile)
			DO Num = 1 TO NumChunks
				InNum = Right(Num, 3, '0')
				ThisFilename = OutFile||'.'||InNum
				InSize = InSize + Stream(ThisFilename, 'c', 'query size')
				CALL CharOut OutFile, CharIn(ThisFilename, 1, ChunkSize)
				CALL LineOut(ThisFilename)
				RC = Stream(ThisFilename, 'c', 'close')
				IF RC <> 'READY' THEN
					SAY 'Error! RC =' RC
				SAY 'Progress:' Num / NumChunks * 100||'%'
			END
		END
	RC = Stream(OutFile, 'c', 'close')
	IF RC <> 'READY:' THEN
		SAY 'Error! RC =' RC

	IF Verify = 1 THEN
		CALL VerifyOutput 'join' InSize
RETURN


CountChunks: Procedure expose OutFile
	NumChunks = 0
	DO FOREVER
		NumChunks = NumChunks + 1
		OutNum = Right(NumChunks, 3, '0')
		CheckFile = OutFile||'.'||OutNum
		IF Stream(CheckFile, 'c', 'query exists') = '' THEN
			DO
				NumChunks = NumChunks - 1
				LEAVE
			END
	END
RETURN NumChunks
*/

VerifyOutput:
	PARSE ARG WhichWay
	IF WhichWay = 'split' THEN
		InSize = Stream(Filename, 'c', 'query size')
	ELSE
		OutSize = Stream(OutFile, 'c', 'query size')
	IF InSize = OutSize THEN
		SAY 'Input and output sizes match!'
	ELSE
		DO
			SAY 'Size(s) of input file(s) is not equal to size(s) of output file(s).'
			SAY 'Input =' InSize||'09'x||'Output =' OutSize
		END
RETURN


Warn:
	IF Stream(OutFile, 'c', 'query exists') <> '' THEN
		DO
			SAY 'File is about to be overwritten:'||'0d0a'x||OutFile||'0d0a'x||'Overwrite?'
			DO UNTIL Response = 'Y' | Response = 'N'
				Response = Translate(SysGetKey(NOECHO))
			END
			IF Response = 'N' THEN
				Proceed = 0
		END
RETURN

